/*
	CreateTemplateFileCmd.cc

	Will Eighteen, 2000: will@opaline.demon.co.uk
*/

#include "CreateTemplateFileCmd.h"
#include "FileAccess.h"
#include "wave/waveclass.h"
#include <MessageDialogManager.h>


CreateTemplateFileCmd::CreateTemplateFileCmd ( char *name, int active, char *filename, long numSamples, long blockLength, int dataType ) : BreakCmd ( name, active )
{
	_name = name;
	strncpy ( _dstFile, filename, BUFSIZE );
	_dstFile [ sizeof ( _dstFile ) - 1 ] = 0;
	_once = 0;
	_numSamples = numSamples;
	_blockLength = blockLength;
	_dataType = dataType;
	_samplesWritten = 0;
	_percentDone = 0;
	_error = 0;
	_defaultValue = 0.0;
}


CreateTemplateFileCmd::~CreateTemplateFileCmd()
{
	// empty
}


void CreateTemplateFileCmd::doCommand()
{
	char text[256];
	int error = 0;
	int percent;

	if ( _done == True )
		return;

	if ( !_once )
	{
		// can we open dst for write?
		FileAccess *dst = new FileAccess ( _dstFile, "w" );

		if ( dst->errNum() != 0 )
		{
			char msg[BUFSIZE<<1];

			error++;
			snprintf ( msg, sizeof ( msg ), "%s\nFile write error: %s", _dstFile, dst->error() );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}

		if ( error )
		{
			_done = True;
			_error = error;
			delete dst;
			return;
		}

		_dst = dst;
		// enforce 2 ms min. timeout
		updateTimeout ( ( int ) ( _numSamples / _blockLength ) + 2 );
		_once = 1;
	}

	int processSamples;
	int count;

	if ( _dataType == 0 )
	{
		int *intArray;
		int value = ( int ) _defaultValue;

		processSamples = ( int ) ( _blockLength / sizeof ( int ) );	// int

		if ( processSamples > ( _numSamples - _samplesWritten ) )
			processSamples = _numSamples - _samplesWritten;

		intArray = new int [ processSamples ];
		_dst->elementSize ( sizeof ( int ) );
		_dst->numElements ( processSamples );

		for ( count = 0; count < processSamples; count++ )
			intArray [ count ] = value;

		_dst->writeF ( intArray );
		delete[] intArray;
	}		
	else
	{
		float *fltArray;
		float fValue = _defaultValue;

		processSamples = ( int ) ( _blockLength / sizeof ( float ) );	// float

		if ( processSamples > ( _numSamples - _samplesWritten ) )
			processSamples = _numSamples - _samplesWritten;

		fltArray = new float [ processSamples ];
		_dst->elementSize ( sizeof ( float ) );
		_dst->numElements ( processSamples );

		for ( count = 0; count < processSamples; count++ )
			fltArray [ count ] = fValue;

		_dst->writeF ( fltArray );
		delete[] fltArray;
	}

	_samplesWritten += processSamples;

	if ( _samplesWritten < _numSamples )
	{
		percent = ( int ) ( ( float ) _samplesWritten / ( float ) _numSamples * 100 );

		if ( _percentDone != percent )
		{
			_percentDone = percent;
			snprintf(text, sizeof ( text ), "%d%% %s", _percentDone, _name);
			text [ sizeof ( text ) - 1 ] = 0;
			updateMessage ( text );
		}
	}
	else
	{	
		delete _dst;
		_done = True;
	}
}
