/*
	LoadStateCmd.cc

	Will Eighteen, 2000-2006: will@opaline.demon.co.uk
*/

#include "wave/waveclass.h"
#include "AppWindow.h"
#include "LoadStateCmd.h"
#include "OpResources.h"
#include <MessageDialogManager.h>
#include <FileAccess.h>
#include <FileSelect.h>
#include "DeletePakCmd.h"


LoadStateCmd::LoadStateCmd ( char *name, int active ) : NoUndoCmd ( name, active )
{
	_name = name;
	_popupRealised = False;
}


void LoadStateCmd::doCommand()
{
	if ( _popupRealised == True )
		return;

	if ( ( wavePak->Count() != 0 ) || ( templatePak->Count() != 0 ) )
	{
		char msg[BUFSIZE];
		strncpy ( msg, "Waves or templates already loaded.\n\
Use 'File/State/Save', 'File/Exit'\nand restart with 'File/State/Load'\n\
or use 'File/Delete' to remove the\ncurrent waves/templates.", sizeof ( msg) );
		msg [ sizeof ( msg ) - 1 ] = 0;
		messageDialogManager->post ( msg );
		return;
	}

	char title[BUFSIZE];
	snprintf ( title, sizeof ( title ), "%s file", _name );
	title [ sizeof ( title ) - 1 ] = 0;
	
	_fileSelect = new FileSelect ( title, TRUE,
								&LoadStateCmd::stateSelectedCallback, NULL,
								&LoadStateCmd::stateCancelledCallback, ( void * ) this );
	_fileSelect->execute();
	_popupRealised = True;
}


void LoadStateCmd::stateSelectedCallback ( void *clientData, char *filename )
{
	LoadStateCmd *obj = ( LoadStateCmd * ) clientData;

	if ( filename[strlen(filename) - 1] != '/' )	// no file specified
		obj->stateSelected ( filename );

	obj->_popupRealised = False;
	delete obj->_fileSelect;
}


void LoadStateCmd::stateSelected ( char *filename )
{
// put 'save state on exit' toggle in options?
	FileAccess *in = new FileAccess ( filename, "r" );
	
	if ( in->errNum() != 0 )
	{
		char msg[BUFSIZE];
		
		snprintf ( msg, sizeof ( msg ), "LoadState error: %s\n", in->error() );
		msg [ sizeof ( msg ) - 1 ] = 0;
		delete in;
		return;
	}
	
	int count = 0;
	int numWaves;
	int numTemplates;
	int nameLength;
	char name[BUFSIZE];
	char syncFile[BUFSIZE];
	char buf[BUFSIZE];
	int error;
	
	in->numElements ( 1 );
	in->elementSize ( sizeof ( int ) );
	
	in->readF ( &numWaves );
	in->readF ( &numTemplates );
	
	if ( numWaves > 0 )
	{
		for ( count = 0; count < numWaves; count++ )
		{
			error = 0;
			in->elementSize ( sizeof ( int ) );
			in->readF ( &nameLength );
			in->elementSize ( nameLength );
			in->readF( &name );
			name [ nameLength ] = 0;				

			FileAccess *chk = new FileAccess ( name, "r+" );
			error = chk->errNum();
			delete chk;

			if ( error != 0 )	// load r/w failed
			{
				FileAccess *chk = new FileAccess ( name, "r" );

				if ( chk->errNum() != 0 )
				{
					snprintf ( buf, sizeof ( buf ), "'%s'\nError: %s\nLoad state failed.", name, chk->error() );
					buf [ sizeof ( buf ) - 1 ] = 0;
					messageDialogManager->post ( buf );	// fatal error

					DeletePakCmd *deletePak = new DeletePakCmd ( "DeleteAll", TRUE );
					deletePak->execute();
					delete deletePak;
					// deleted pak contents, now remove the paks...
					wavePak->Release();	// delete list of wavePak structures
					templatePak->Release();	// delete list of templatePak structures
					// don't proceed with missing files
					delete chk;
					delete in;
					return;
				}
				else
					error = 0;	// read-only, ok
				
				delete chk;
			}
			// note: doesn't determine if file perms have changed since state save

			if ( !error )
			{
				in->elementSize ( sizeof ( *wavePak ) );
				WavePak temp;
				in->readF ( &temp );
				wavePak->Add ( temp );

				Wave *pWave = new Wave;
				pWave->LoadHeader ( ( char * ) name );
				(*wavePak)[count].pWave = pWave;

				// reset pointers to pak objects, filled in later
				// by invocation of relevant class instance
				(*wavePak)[count].pIntArray = NULL;
				(*wavePak)[count].graphCmd = NULL;
				(*wavePak)[count].graphNum = 0;
				(*wavePak)[count].waveDataCmd = NULL;
				(*wavePak)[count].infoNum = 0;
				MixPak tempmix;
				tempmix.waveNum = pWave->waveNum();
				tempmix.start = 0;
				tempmix.end = 0;
				mixPak->Add ( tempmix );
			}
		}
	}
		
	if ( numTemplates > 0 )
	{
		for ( count = 0; count < numTemplates; count++ )
		{
			in->elementSize ( sizeof ( int ) );
			in->readF ( ( void * ) &nameLength );
			in->elementSize ( nameLength );
			in->readF ( &name );
			name [ nameLength ] = 0;				

			FileAccess *chk = new FileAccess ( name, "r" );
			error = chk->errNum();
			delete chk;

			if ( error != 0 )
			{
				snprintf ( buf, sizeof ( buf ), "'%s'\nError: %s\nLoad state failed.", name, chk->error() );
				buf [ sizeof ( buf ) - 1 ] = 0;
				messageDialogManager->post ( buf );	// fatal error

				DeletePakCmd *deletePak = new DeletePakCmd ( "DeleteAll", TRUE );
				deletePak->execute();
				delete deletePak;
				// deleted pak contents, now remove the paks...
				wavePak->Release();	// delete list of wavePak structures
				templatePak->Release();	// delete list of templatePak structures
				delete chk;
				delete in;
				return;
				// don't proceed with missing files
			}
			else
			{
				in->elementSize ( sizeof ( int ) );
				in->readF ( ( void * ) &nameLength );
				in->elementSize ( nameLength );
				in->readF ( &syncFile );
				syncFile [ nameLength ] = 0;				
				in->elementSize ( sizeof ( *templatePak ) );
				TemplatePak temp;
				in->readF ( &temp );
				templatePak->Add ( temp );
				
				strncpy ( (*templatePak)[count].name, name, sizeof ( (*templatePak)[count].name, name ) );
				strncpy ( (*templatePak)[count].syncFile, syncFile, sizeof ( (*templatePak)[count].syncFile ) );

				(*templatePak)[count].graphIntCmd = NULL;
				(*templatePak)[count].graphFltCmd = NULL;
				(*templatePak)[count].graphNum = 0;
				(*templatePak)[count].pIntArray = NULL;
				(*templatePak)[count].pFloatArray = NULL;
				(*templatePak)[count].infoNum = 0;
			}
		}
	}

	delete in;
}


void LoadStateCmd::stateCancelledCallback ( void *clientData )
{
	LoadStateCmd *obj = ( LoadStateCmd * ) clientData;

	obj->_popupRealised = False;
	delete obj->_fileSelect;
}

