/*
	ViewWaveCmd.cc

	Will Eighteen, 2001, 2006: will@opaline.demon.co.uk
*/

#include <DMenuCmd.h>
#include <GraphFltCmd.h>
#include "OpResources.h"
#include "ViewWaveCmd.h"
#include "AppWindow.h"
#include "MessageDialogManager.h"


ViewWaveCmd::ViewWaveCmd ( const char *name, int active ) : NoUndoCmd ( name, active )
{
	_currentSamples = 0;
	_viewMenuRealised = False;
}


void ViewWaveCmd::doCommand()
{
	unsigned int n;
	unsigned int maxNameLength = 0;
	
	if ( (*wavePak)[0].pWave->first() == NULL )
		return;
		
	if ( _viewMenuRealised == True )
		return;

	_arrayItems = wavePak->Count();

	for ( n = 0; n < _arrayItems; n++ )
		if ( strlen ( (*wavePak)[n].pWave->name() ) > maxNameLength )
			maxNameLength = strlen ( (*wavePak)[n].pWave->name() );

	n = 0;
	maxNameLength++;
	
	if ( maxNameLength < 5 )
		maxNameLength = 5;	// accomodate 'quit' at a minimum
		
	_arrayItems++;
	
	char strings [ _arrayItems ] [ maxNameLength ];

	for ( n = 0; n < ( _arrayItems - 1 ); n++ )
		strcpy ( (char *) &strings [ n ], (*wavePak)[n].pWave->name() );

	strcpy ( strings [ n ], "Quit" );

	_dMenu = new DMenuCmd ( "Wave view menu", TRUE, FALSE,
								&ViewWaveCmd::dMenuSelectionCallback,
								( void * ) this, "Current wave files",
								_arrayItems, maxNameLength, ( char * ) &strings );
	_viewMenuRealised = True;
	_dMenu->execute();
}


void ViewWaveCmd::dMenuSelectionCallback ( void *clientData, unsigned int selectedItemNum, char *label )
{
	ViewWaveCmd *obj = ( ViewWaveCmd * ) clientData;	// dummy obj

	// Since the menu is static with reference to the name strings
	// (quitOnSelect is false), the reference may be invalid; check by name.

	if ( strcmp ( label, "Quit" ) == 0 )
	{
		obj->_viewMenuRealised = False;			// menu will be popped down
		delete obj->_dMenu;
	}
	else
	{
		int count, found;
	
		// scan base list for returned name
		for ( count = 0; count < wavePak->Count(); count++ )
		{
			found = 0;

			if ( strcmp ( label, (*wavePak)[count].pWave->name() ) == 0 )
			{
				if ( (*wavePak)[count].fLock == ACTIVE )
				{
					char msg[BUFSIZE];
					snprintf ( msg, sizeof ( msg ), "%s\nFile in use!", (*wavePak)[count].pWave->name() );
					msg [ sizeof ( msg ) - 1 ] = 0;
					messageDialogManager->post ( msg );
					return;
				}

				obj->viewWave ( (*wavePak)[count].pWave->waveNum() );
				found = 1;
				count = wavePak->Count();
			}
		}
		
		if ( !found && mixwaveResources->enableMessagePopups() == True )
		{
			char msg[BUFSIZE];
			snprintf ( msg, sizeof ( msg ), "Error: %s\nFile not loaded.", label );
			msg [ sizeof ( msg ) - 1 ] = 0;
			messageDialogManager->post ( msg );
		}
	}
}


void ViewWaveCmd::viewWave ( unsigned int selectedItemNum )
{
	if ( selectedItemNum == 0 )	// WM 'X' activated in menu popup
	{
		_viewMenuRealised = False;
		return;
	}

	if ( (*wavePak)[selectedItemNum-1].filePerm != 0 )
	{
		if ( mixwaveResources->enableMessagePopups() == True )
		{
			char buf[BUFSIZE];
			snprintf ( buf, sizeof ( buf ), "%s\nWarning - file is read-only; changes will be discarded.",
						(*wavePak)[selectedItemNum-1].pWave->name() );
			buf [ sizeof ( buf ) - 1 ] = 0;
			messageDialogManager->post ( buf );
		}
	}
	
	for ( int count = 0; count < wavePak->Count(); count++ )
		if ( (*wavePak)[count].graphNum == selectedItemNum )
			return;	// wave editor is already created for this wave

	if ( (*wavePak)[selectedItemNum-1].pWave->numChannels() != 1 )
	{
		char buf[BUFSIZE];
		snprintf ( buf, sizeof ( buf ), "%s\nchannels = %u: only mono files can be viewed.",
					(*wavePak)[selectedItemNum-1].pWave->name(),
					(*wavePak)[selectedItemNum-1].pWave->numChannels() );

		buf [ sizeof ( buf ) - 1 ] = 0;
		messageDialogManager->post ( buf );
		_viewMenuRealised = False;
		return;
	}

	if ( (*wavePak)[selectedItemNum-1].pWave->dataLength() == 0 )
	{
		char buf[BUFSIZE];
		snprintf ( buf, sizeof ( buf ), "%s\n No data in file (use 'Edit/Copy' to insert data).",
					(*wavePak)[selectedItemNum-1].pWave->name() );
		buf [ sizeof ( buf ) - 1 ] = 0;
		messageDialogManager->post ( buf );
		_viewMenuRealised = False;
		return;
	}		

	int bytesPerSample = (*wavePak)[selectedItemNum - 1].pWave->bitsPerSample() / 8;

	(*wavePak)[selectedItemNum - 1].pWave->filePos = (*wavePak)[selectedItemNum-1].dataOffset * bytesPerSample;
	(*wavePak)[selectedItemNum-1].fLock = ACTIVE;
	(*wavePak)[selectedItemNum-1].pWave->LoadBlock ( (*wavePak)[selectedItemNum-1].pWave->filePos, (*wavePak)[selectedItemNum-1].intArraySize * bytesPerSample );
	(*wavePak)[selectedItemNum-1].fLock = INACTIVE;

	createIntDataArray ( selectedItemNum, (*wavePak)[selectedItemNum-1].intArraySize * bytesPerSample );

	GraphIntCmd *graphCmd = new GraphIntCmd ( (*wavePak)[selectedItemNum-1].pWave->name(),
										TRUE, &ViewWaveCmd::viewWaveCmdCallback,
										mixwaveResources->numDigits(),
										(*wavePak)[selectedItemNum-1].pIntArray,
										(*wavePak)[selectedItemNum-1].dataOffset,
										_currentSamples,
										( (*wavePak)[selectedItemNum-1].pWave->totalSamples() ),
										(*wavePak)[selectedItemNum-1].pWave->waveNum(),
										( void * ) this );

	(*wavePak)[selectedItemNum-1].graphCmd = graphCmd;
	// save the current view ID for the wave
	(*wavePak)[selectedItemNum-1].graphNum = (*wavePak)[selectedItemNum-1].pWave->waveNum();
	graphCmd->setDatumValueBounds ( mixwaveResources->minY(), mixwaveResources->maxY() );

	if ( mixwaveResources->minY() < 0 )	// set by WaveDefaultsCmd bitsPerSample
		graphCmd->centreZero ( True );	// set to signed data type
	else
		graphCmd->centreZero ( False );

	// get the user-defined window size string into ints
	int width, height;
	sscanf ( mixwaveResources->viewWindowSize(), "%dx%d", &height, &width );
	graphCmd->setWindowSize ( height, width );
	graphCmd->execute();
	graphCmd->setSelection ( (*wavePak)[selectedItemNum-1].startSel, (*wavePak)[selectedItemNum-1].endSel );

	WaveDataCmd *waveDataCmd = new WaveDataCmd ( (*wavePak)[selectedItemNum-1].pWave->name(),
										TRUE, &ViewWaveCmd::waveDataCmdCallback,								
										(*wavePak)[selectedItemNum-1].pWave,
										(*wavePak)[selectedItemNum-1].pIntArray,
										mixwaveResources->numDigits(),
										(*wavePak)[selectedItemNum-1].dataOffset,
										_currentSamples,
										( void * ) this );

	// save the current wave data object pointer
	(*wavePak)[selectedItemNum-1].waveDataCmd = waveDataCmd;
	waveDataCmd->setDatumValueBounds ( mixwaveResources->minY(), mixwaveResources->maxY() );
	waveDataCmd->setFPDigits ( mixwaveResources->fpDigits() );
	waveDataCmd->execute();
}


void ViewWaveCmd::viewWaveCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, long yCurrent, Boolean reload,
								unsigned waveNum, unsigned callbackType )
{
	// valid callback types: '0' report, '1' ok, '2' cancel.
	ViewWaveCmd *obj = ( ViewWaveCmd * ) clientData;
	GraphIntCmd *gc = ( GraphIntCmd * ) cmd;

	if ( (*wavePak)[waveNum-1].fLock == INACTIVE )
	{
		obj->viewWaveCmdCallbackFinished ( gc, start, end, selStart, selEnd,
											xCurrent, yCurrent,
											reload, waveNum, callbackType );
	}
}


void ViewWaveCmd::waveDataCmdCallback ( void *cmd, void *clientData, long start, long end,
								long selStart, long selEnd, long xCurrent, long yCurrent, Boolean reload,
								unsigned waveNum, unsigned callbackType )
{
	ViewWaveCmd *obj = ( ViewWaveCmd * ) clientData;
//	WaveDataCmd *wd = ( WaveDataCmd * ) cmd;	// unused

	if ( (*wavePak)[waveNum-1].fLock == INACTIVE )
	{
		obj->viewWaveCmdCallbackFinished ( (*wavePak)[waveNum-1].graphCmd, start, end, selStart, selEnd,
											xCurrent, yCurrent, reload, waveNum, callbackType );
	}
	else
		return;

	// 'output' from data obj dependent on GraphCmd obj; used to drive GC 'input'
	// if data has been changed here, reflect this in the GraphCmd obj

	if ( reload == True )
		(*wavePak)[waveNum-1].dataOffset = start;	// start posn of new data

	if ( (*wavePak)[waveNum-1].graphCmd->redisplay ( start, end, selStart, selEnd, xCurrent,
														(*wavePak)[waveNum-1].dataOffset ) != 0 )
	{
		char msg[BUFSIZE];
		snprintf ( msg, sizeof ( msg ), "%s\nViewWaveCmd view update failed. No data to display.",
					(*wavePak)[waveNum-1].pWave->name() );
		msg [ sizeof ( msg ) - 1 ] = 0;
	}
}


void ViewWaveCmd::viewWaveCmdCallbackFinished ( GraphIntCmd *gc, long start, long end,
								long selStart, long selEnd, long xCurrent, long yCurrent, Boolean reload,
								unsigned waveNum, unsigned callbackType )
{
// debug - if xCurrent isn't valid for the array, yCurrent causes a seg fault
//printf("ViewWaveCmd::viewWaveCmdCallbackFinished\n");
//printf("start, end, selStart, selEnd, xCurrent, yCurrent\n");
//printf("%ld %ld %ld %ld %ld %ld\n", start, end, selStart, selEnd, xCurrent, yCurrent);
//printf("reload status %d callbackType %u\n", reload, callbackType);
	if ( (*wavePak)[waveNum-1].fLock == ACTIVE )
	{
		(*wavePak)[waveNum-1].graphCmd->redisplay (
											(*wavePak)[waveNum-1].startRange,
											(*wavePak)[waveNum-1].endRange,
											(*wavePak)[waveNum-1].startSel,
											(*wavePak)[waveNum-1].endSel,
											(*wavePak)[waveNum-1].xCurrent,
											(*wavePak)[waveNum-1].dataOffset );
		return;
	}

	int dataChanged;
	int bytesPerSample = (*wavePak)[waveNum-1].pWave->bitsPerSample() / 8;

	dataChanged = (*wavePak)[waveNum-1].graphCmd->dataChanged();
	dataChanged += (*wavePak)[waveNum-1].waveChanged;	// if set externally
	(*wavePak)[waveNum-1].waveChanged = 0;

	long length;

	if ( ( dataChanged != 0 ) && ( (*wavePak)[waveNum-1].filePerm == 0 ) )
	{	// fileperm 0 = r/w, 1 = read, -1 = invalid (see AppWindow.cc\loadWave)
		int offset;

		if ( (*wavePak)[waveNum-1].graphCmd->startData() == 0 )
			offset = (*wavePak)[waveNum-1].startRange;
		else
			offset = 0;

		length = (*wavePak)[waveNum-1].endRange - (*wavePak)[waveNum-1].startRange + 1;
		long dataSize = length * bytesPerSample;

		if ( !allocDataBlock ( dataSize ) )
		{
			dataChanged = 0;
			
			char buf[BUFSIZE];
			snprintf ( buf, sizeof ( buf ), "%s\nData block allocation failed in\n\
ViewWaveCmd::viewWaveCmdCallbackFinished\nReduce selected region size - request \
for\n%d sample space failed.\nOperation abandoned.", (*wavePak)[waveNum-1].pWave->name(), dataSize );
			buf [ sizeof ( buf ) - 1 ] = 0;
			messageDialogManager->post ( buf );
			return;
		}

		switch ( bytesPerSample )
		{
			case 1:
				break;

			case 2:
				for ( long count = 0; count < length; count++ )
				{

					_pData[count*2] = ( char )(
					(*wavePak)[waveNum-1].pIntArray[count + offset] & 0x000000ff );
					_pData[(count*2)+1] = ( char )(
					(*wavePak)[waveNum-1].pIntArray[count + offset] >> 8 );
				}

				break;

			case 3:
				break;
		}

		// since file length is unaltered, just contents, write the block out
		(*wavePak)[waveNum-1].fLock = ACTIVE;
		(*wavePak)[waveNum-1].pWave->SaveBlock ( (*wavePak)[waveNum-1].pWave->name(),
												(*wavePak)[waveNum-1].startRange*bytesPerSample,
												dataSize, _pData );
		(*wavePak)[waveNum-1].fLock = INACTIVE;
	}
	// 'output' used to drive 'inputs' to data objects
	long blockSize, max, min;

	(*wavePak)[waveNum-1].startRange = start;
	(*wavePak)[waveNum-1].endRange = end;
	(*wavePak)[waveNum-1].startSel = selStart;
	(*wavePak)[waveNum-1].endSel = selEnd;
	(*wavePak)[waveNum-1].xCurrent = xCurrent;
	(*wavePak)[waveNum-1].yCurrent = yCurrent;

	if ( callbackType != 0 )	// re-enable view popup on 'ok' or 'cancel'
	{
		(*wavePak)[waveNum-1].intArraySize = end - start + 1;
		(*wavePak)[waveNum-1].graphNum = 0;

		// if the pointer's non-null the time data view hasn't been cancelled
		if ( (*wavePak)[waveNum-1].waveDataCmd != NULL )
		{
			(*wavePak)[waveNum-1].waveDataCmd->cancel();
			delete (*wavePak)[waveNum-1].waveDataCmd;
			(*wavePak)[waveNum-1].waveDataCmd = NULL;
		}

		(*wavePak)[waveNum-1].dataOffset = (*wavePak)[waveNum-1].startRange;
		delete (*wavePak)[waveNum-1].graphCmd;
		(*wavePak)[waveNum-1].graphCmd = NULL;
	}

	if ( callbackType == 2 )	// clear range and selection on 'Cancel'
	{
		(*wavePak)[waveNum-1].startRange = 0;
		(*wavePak)[waveNum-1].endRange = 0;
		(*wavePak)[waveNum-1].startSel = 0;
		(*wavePak)[waveNum-1].endSel = 0;
		(*wavePak)[waveNum-1].xCurrent = 0;
		(*wavePak)[waveNum-1].yCurrent = 0;
		(*wavePak)[waveNum-1].dataOffset = 0;
		return;
	}
	else
	{
		if ( callbackType == 1 )
			return;	// 'Ok' - just keep pointers to array selection and quit
	}
	
	if ( reload == True )
	{
		(*wavePak)[waveNum-1].waveChanged = False;
		min = start;

		if ( end < min )
			min = end;
		
		if ( selStart != selEnd )
		{
			if ( selStart < min )
				min = selStart;
	
			if ( selEnd < min )
				min = selEnd;
		}
		
		max = start;
		
		if ( end > max )
			max = end;

		if ( selStart != selEnd )
		{
			if ( selStart > max )
				max = selStart;
	
			if ( selEnd > max )
				max = selEnd;
		}

		(*wavePak)[waveNum-1].pWave->filePos = min * bytesPerSample;
		blockSize = ( max - min + 1 ) * bytesPerSample;
		(*wavePak)[waveNum-1].fLock = ACTIVE;
		(*wavePak)[waveNum-1].pWave->LoadBlock ( (*wavePak)[waveNum-1].pWave->filePos, blockSize );
		(*wavePak)[waveNum-1].fLock = INACTIVE;
		createIntDataArray ( waveNum, ( max - min + 1 ) * bytesPerSample );
		gc->setDataSource ( (*wavePak)[waveNum-1].pIntArray, max - min + 1, (*wavePak)[waveNum-1].pWave->totalSamples() );
	}

	// now drive the input to graph-dependent waveDataCmd, if it exists
	if ( (*wavePak)[waveNum-1].waveDataCmd != NULL )
	{
		(*wavePak)[waveNum-1].waveDataCmd->setArrayPtr( (*wavePak)[waveNum-1].pIntArray );
		(*wavePak)[waveNum-1].waveDataCmd->setFields (
						(*wavePak)[waveNum-1].startRange,
						(*wavePak)[waveNum-1].endRange,
						(*wavePak)[waveNum-1].startSel,
						(*wavePak)[waveNum-1].endSel,
						(*wavePak)[waveNum-1].xCurrent,
						(*wavePak)[waveNum-1].yCurrent,
						(*wavePak)[waveNum-1].pWave->totalSamples() );
	}

	// drive associated template view, if it exists
	if ( mixwaveResources->syncTemplateLength() == True )
	{
		unsigned templateNum = (*wavePak)[waveNum-1].templateNum;
		float syncRatio = (*templatePak)[templateNum-1].syncRatio;
		
		if ( mixwaveResources->linkViews() == False	 )	// wave drives template
		{
			if ( (*templatePak)[(*wavePak)[waveNum-1].templateNum-1].dataType == 0 )
			{
				// get template for template num
				if ( (*templatePak)[(*wavePak)[waveNum-1].templateNum-1].graphIntCmd != NULL )
				{	// template view exists
					(*templatePak)[templateNum-1].startRange = ( long ) ( ( ( float ) start / syncRatio ) + 0.5 );
					(*templatePak)[templateNum-1].endRange = ( long ) ( ( ( float ) end / syncRatio ) + 0.5 );
					(*templatePak)[templateNum-1].startSel = ( long ) ( ( ( float ) selStart / syncRatio ) + 0.5 );
					(*templatePak)[templateNum-1].endSel = ( long ) ( ( ( float ) selEnd / syncRatio ) + 0.5 );
					(*templatePak)[templateNum-1].xCurrent = ( long ) ( ( ( float ) xCurrent / syncRatio ) + 0.5 );
					(*templatePak)[templateNum-1].dataOffset =  ( long ) ( ( ( float ) start / syncRatio ) + 0.5 );
	
					(*templatePak)[(*wavePak)[waveNum-1].templateNum-1].graphIntCmd->redisplay (
								(*templatePak)[templateNum-1].startRange,
								(*templatePak)[templateNum-1].endRange,
								(*templatePak)[templateNum-1].startSel,
								(*templatePak)[templateNum-1].endSel,
								(*templatePak)[templateNum-1].xCurrent,
								(*templatePak)[templateNum-1].dataOffset );
				}
			}
			else
			{
				if ( (*templatePak)[(*wavePak)[waveNum-1].templateNum-1].graphFltCmd != NULL )
				{	// template view exists
					(*templatePak)[templateNum-1].startRange = ( long ) ( ( ( float ) start / syncRatio ) + 0.5 );
					(*templatePak)[templateNum-1].endRange = ( long ) ( ( ( float ) end / syncRatio ) + 0.5 );
					(*templatePak)[templateNum-1].startSel = ( long ) ( ( ( float ) selStart / syncRatio ) + 0.5 );
					(*templatePak)[templateNum-1].endSel = ( long ) ( ( ( float ) selEnd / syncRatio ) + 0.5 );
					(*templatePak)[templateNum-1].xCurrent = ( long ) ( ( ( float ) xCurrent / syncRatio ) + 0.5 );
					(*templatePak)[templateNum-1].dataOffset =  ( long ) ( ( ( float ) start / syncRatio ) + 0.5 );
	
					(*templatePak)[(*wavePak)[waveNum-1].templateNum-1].graphFltCmd->redisplay (
								(*templatePak)[templateNum-1].startRange,
								(*templatePak)[templateNum-1].endRange,
								(*templatePak)[templateNum-1].startSel,
								(*templatePak)[templateNum-1].endSel,
								(*templatePak)[templateNum-1].xCurrent,
								(*templatePak)[templateNum-1].dataOffset );
				}
			}
		}
	}

	if ( _pData )
		freeDataBlock();
}


void ViewWaveCmd::createIntDataArray ( unsigned waveNum, long blockSize )
{
	char *pData;
	unsigned int i, j;
	int arrayVal;
	int bytesPerSample = (*wavePak)[waveNum - 1].pWave->bitsPerSample() / 8;
	long currentSample;

	delete[] ( *wavePak )[ waveNum - 1 ].pIntArray;	// delete original array pointer
	pData = (*wavePak)[waveNum-1].pWave->data();

	if (((*wavePak)[waveNum-1].pWave->dataLength() < blockSize) || (blockSize == 0))
		_currentSamples = (*wavePak)[waveNum-1].pWave->totalSamples();
	else
		_currentSamples = blockSize / bytesPerSample;

	// loaded block size is preserved so wave editor can be re-popped up after 'ok' 
	// or 'cancel', avoiding wave reload with default mixwaveResources->blockLength()
	(*wavePak)[waveNum-1].intArraySize = _currentSamples;
	(*wavePak)[waveNum-1].pIntArray = new int [_currentSamples];

	switch ( bytesPerSample )
	{
		case 1:
			break;

		case 2:	// convert pWave data to 16-bit format
			for ( currentSample = 0; currentSample < _currentSamples; currentSample++ )
			{
				i = (unsigned char) pData[currentSample*bytesPerSample];
				j = (unsigned char) pData[currentSample*bytesPerSample+1];
				arrayVal = i + 256 * j;

				if ( arrayVal > mixwaveResources->maxY() )
					arrayVal -= ( mixwaveResources->maxY() - mixwaveResources->minY() + 1);

				(*wavePak)[waveNum-1].pIntArray[currentSample] = arrayVal;
			}

			break;

		case 3:
			break;
	}
}


Boolean ViewWaveCmd::allocDataBlock ( long size )
{
	freeDataBlock();

	_pData = new char [size];

	if ( _pData )
		return True;
	else
		return False;
}


void ViewWaveCmd::freeDataBlock()
{
	if ( _pData )
		delete[] _pData;

	_pData = 0;
}
