/*
 MenuWindow.cc - add a MenuBar to MainWindow

 Douglas A. Young, 'Object-oriented Programming with C++ and OSF/Motif'
 2nd ed., Prentice-Hall.
*/


#include "MenuWindow.h"
#include "MenuBar.h"


MenuWindow::MenuWindow ( const char *name ) : MainWindow ( name )
{
	_menuBar = NULL;
}


void MenuWindow::initialize()
{
	MainWindow::initialize();	// call base class for work area
	_menuBar = new MenuBar ( "menubar", _main );
	XtVaSetValues ( _main, XmNmenuBar, _menuBar->baseWidget(), NULL );
	
	// call derived class hook to add panes to the menu
	createMenuPanes();
	_menuBar->manage();
}


MenuWindow::~MenuWindow()
{
	delete _menuBar;
}

	
	
	
/*
  Notes:
  
  The class initialize() function instantiates a MenuBar object and specifies the
  MenuBar object's base widget as the value of the XmMainWindow widget's
  XmNmenuBar resource.
  
  createMenuPanes() allows derived classes to add panes to the menu bar before it
  is managed. This function must be implemented by all derived classes.
*/
